@@ -1,6 +1,6 @@
 """
-Module: electrons.phase_recon
------------------------------
+Module: ptyrodactyl.electrons.phase_recon
+------------------------------------------
 Inverse reconstruction algorithms for electron ptychography.
 
 This module contains functions for reconstructing sample potentials from
@@ -14,19 +14,11 @@ Functions
     Performs single-slice ptychography reconstruction
 - `single_slice_poscorrected`:
     Performs single-slice reconstruction with position correction
-- `single_slice_multi_modal`:
+single_slice_multi_modal
     Performs single-slice reconstruction with multi-modal probe
-- `multi_slice_multi_modal`:
+multi_slice_multi_modal
     Performs multi-slice reconstruction with multi-modal probe
 
-Internal Functions
-------------------
-These functions are not exported and are used internally by the module.
-
-- `_get_optimizer`:
-    Returns an optimizer instance for the specified optimization algorithm
-
-
 Notes
 -----
 All reconstruction functions use JAX-compatible optimizers and support
@@ -34,15 +26,24 @@ automatic differentiation. The functions are designed to work with
 experimental data and can handle various noise levels and experimental
 conditions. Input data should be properly preprocessed and validated
 using the factory functions from electron_types module.
+
+Internal functions (not exported):
+- _get_optimizer: Returns an optimizer instance for the specified optimization algorithm
 """
 
 import jax
 import jax.numpy as jnp
-from beartype import beartype as typechecker
 from beartype.typing import Any, Dict, Optional, Tuple, Union
-from jaxtyping import Array, Complex, Float, jaxtyped
+from jaxtyping import Array, Complex, Float, Int
 
 import ptyrodactyl.tools as ptt
+from ptyrodactyl._decorators import beartype as typechecker
+from ptyrodactyl._decorators import jaxtyped
+
+from .electron_types import (STEM4D, CalibratedArray, ProbeModes,
+                             make_calibrated_array, scalar_float, scalar_int,
+                             scalar_numeric)
+from .simulations import stem_4d
 
 jax.config.update("jax_enable_x64", True)
 
@@ -52,11 +53,6 @@ OPTIMIZERS: Dict[str, ptt.Optimizer] = {
     "rmsprop": ptt.Optimizer(ptt.init_rmsprop, ptt.rmsprop_update),
 }
 
-from .electron_types import (CalibratedArray, ProbeModes,
-                             make_calibrated_array, scalar_float, scalar_int,
-                             scalar_numeric)
-from .simulations import stem_4D
-
 
 @typechecker
 def _get_optimizer(optimizer_name: str) -> ptt.Optimizer:
@@ -67,13 +63,10 @@ def _get_optimizer(optimizer_name: str) -> ptt.Optimizer:
 
 @jaxtyped(typechecker=typechecker)
 def single_slice_ptychography(
-    experimental_4dstem: Float[Array, "P H W"],
+    experimental_data: STEM4D,
     initial_potential: CalibratedArray,
     initial_beam: CalibratedArray,
-    pos_list: Float[Array, "P 2"],
     slice_thickness: scalar_numeric,
-    voltage_kV: scalar_numeric,
-    calib_ang: scalar_float,
     save_every: Optional[scalar_int] = 10,
     num_iterations: Optional[scalar_int] = 1000,
     learning_rate: Optional[scalar_float] = 0.001,
@@ -85,60 +78,51 @@ def single_slice_ptychography(
     Complex[Array, "H W S"],
     Complex[Array, "H W S"],
 ]:
-    """
-    Description
-    -----------
-    Single Slice Ptychography where the electrostatic potential
-    slice and the beam guess are of the same size.
+    """Single Slice Ptychography where the electrostatic potential slice and the beam guess are of the same size.
 
     Parameters
     ----------
-    - `experimental_4dstem` (Float[Array, "P H W"]):
-        Experimental 4D-STEM data.
-    - `initial_potential` (pte.CalibratedArray):
+    experimental_data : STEM4D
+        Experimental 4D-STEM data PyTree containing diffraction patterns,
+        scan positions, and calibration information.
+    initial_potential : CalibratedArray
         Initial guess for potential slice.
-    - `initial_beam` (pte.CalibratedArray):
+    initial_beam : CalibratedArray
         Initial guess for electron beam.
-    - `pos_list` (Float[Array, "P 2"]):
-        List of probe positions.
-    - `slice_thickness` (scalar_numeric):
+    slice_thickness : scalar_numeric
         Thickness of each slice.
-    - `voltage_kV` (scalar_numeric):
-        Accelerating voltage.
-    - `calib_ang` (scalar_float):
-        Calibration in angstroms.
-    - `save_every` (scalar_int):
-        Save every nth iteration.
-        Optional, default is 10.
-    - `num_iterations` (scalar_int):
-        Number of optimization iterations.
-        Optional, default is 1000.
-    - `learning_rate` (scalar_float):
-        Learning rate for optimization.
-        Optional, default is 0.001.
-    - `loss_type` (str):
-        Type of loss function to use.
-        Optional, default is "mse".
-    - `optimizer_name` (str):
-        Name of optimizer to use.
-        Optional, default is "adam".
+    save_every : scalar_int, optional
+        Save every nth iteration. Default is 10.
+    num_iterations : scalar_int, optional
+        Number of optimization iterations. Default is 1000.
+    learning_rate : scalar_float, optional
+        Learning rate for optimization. Default is 0.001.
+    loss_type : str, optional
+        Type of loss function to use. Default is "mse".
+    optimizer_name : str, optional
+        Name of optimizer to use. Default is "adam".
 
     Returns
     -------
-    - `pot_slice` (pte.CalibratedArray):
-        Optimized potential slice.
-    - `beam` (pte.CalibratedArray):
-        Optimized electron beam.
-    - `intermediate_potslice` (Complex[Array, "H W S"]):
-        Intermediate potential slices.
-    - `intermediate_beam` (Complex[Array, "H W S"]):
-        Intermediate electron beams.
+    tuple of (CalibratedArray, CalibratedArray, Complex[Array, "H W S"], Complex[Array, "H W S"])
+        - pot_slice : CalibratedArray
+            Optimized potential slice.
+        - beam : CalibratedArray
+            Optimized electron beam.
+        - intermediate_potslice : Complex[Array, "H W S"]
+            Intermediate potential slices.
+        - intermediate_beam : Complex[Array, "H W S"]
+            Intermediate electron beams.
     """
+    experimental_4dstem: Float[Array, "P H W"] = experimental_data.data
+    pos_list: Float[Array, "P 2"] = experimental_data.scan_positions
+    voltage_kV: Float[Array, " "] = experimental_data.voltage_kV
+    calib_ang: Float[Array, " "] = experimental_data.real_space_calib
 
-    def forward_fn(
+    def _forward_fn(
         pot_slice: Complex[Array, "H W"], beam: Complex[Array, "H W"]
     ) -> Float[Array, "P H W"]:
-        return stem_4D(
+        stem4d_result = stem_4d(
             pot_slice[None, ...],
             beam[None, ...],
             pos_list,
@@ -146,13 +130,16 @@ def single_slice_ptychography(
             voltage_kV,
             calib_ang,
         )
+        return stem4d_result.data
 
-    loss_func = ptt.create_loss_function(forward_fn, experimental_4dstem, loss_type)
+    loss_func: Any = ptt.create_loss_function(
+        _forward_fn, experimental_4dstem, loss_type
+    )
 
     @jax.jit
-    def loss_and_grad(
+    def _loss_and_grad(
         pot_slice: Complex[Array, "H W"], beam: Complex[Array, "H W"]
-    ) -> Tuple[Float[Array, ""], Dict[str, Complex[Array, "H W"]]]:
+    ) -> Tuple[Float[Array, " "], Dict[str, Complex[Array, "H W"]]]:
         loss, grads = jax.value_and_grad(loss_func, argnums=(0, 1))(pot_slice, beam)
         return loss, {"pot_slice": grads[0], "beam": grads[1]}
 
@@ -168,17 +155,17 @@ def single_slice_ptychography(
         beam = jnp.fft.ifft2(initial_beam.data_array)
 
     @jax.jit
-    def update_step(
+    def _update_step(
         pot_slice: Complex[Array, "H W"],
         beam: Complex[Array, "H W"],
         pot_slice_state: Any,
         beam_state: Any,
     ) -> Tuple[
-        Complex[Array, "H W"], Complex[Array, "H W"], Any, Any, Float[Array, ""]
+        Complex[Array, "H W"], Complex[Array, "H W"], Any, Any, Float[Array, " "]
     ]:
-        loss: Float[Array, ""]
+        loss: Float[Array, " "]
         grads: Dict[str, Complex[Array, "H W"]]
-        loss, grads = loss_and_grad(pot_slice, beam)
+        loss, grads = _loss_and_grad(pot_slice, beam)
         pot_slice, pot_slice_state = optimizer.update(
             pot_slice, grads["pot_slice"], pot_slice_state, learning_rate
         )
@@ -205,14 +192,14 @@ def single_slice_ptychography(
     )
 
     for ii in range(num_iterations):
-        loss: Float[Array, ""]
-        pot_slice, beam, pot_slice_state, beam_state, loss = update_step(
+        loss: Float[Array, " "]
+        pot_slice, beam, pot_slice_state, beam_state, loss = _update_step(
             pot_slice, beam, pot_slice_state, beam_state
         )
 
         if ii % save_every == 0:
             print(f"Iteration {ii}, Loss: {loss}")
-            saver: scalar_int = jnp.floor(ii / save_every)
+            saver: Int[Array, ""] = jnp.floor(ii / save_every).astype(jnp.int32)
             intermediate_potslice = intermediate_potslice.at[:, :, saver].set(pot_slice)
             intermediate_beam = intermediate_beam.at[:, :, saver].set(beam)
 
@@ -234,13 +221,10 @@ def single_slice_ptychography(
 
 @jaxtyped(typechecker=typechecker)
 def single_slice_poscorrected(
-    experimental_4dstem: Float[Array, "P H W"],
+    experimental_data: STEM4D,
     initial_potential: CalibratedArray,
     initial_beam: CalibratedArray,
-    initial_pos_list: Float[Array, "P 2"],
     slice_thickness: scalar_numeric,
-    voltage_kV: scalar_numeric,
-    calib_ang: scalar_float,
     save_every: Optional[scalar_int] = 10,
     num_iterations: Optional[scalar_int] = 1000,
     learning_rate: Optional[Union[scalar_float, Float[Array, "2"]]] = 0.01,
@@ -254,69 +238,62 @@ def single_slice_poscorrected(
     Complex[Array, "H W S"],
     Float[Array, "P 2 S"],
 ]:
-    """
-    Description
-    -----------
-    Create and run an optimization routine for 4D-STEM reconstruction with position correction.
+    """Create and run an optimization routine for 4D-STEM reconstruction with position correction.
 
     Parameters
     ----------
-    - `experimental_4dstem` (Float[Array, "P H W"]):
-        Experimental 4D-STEM data.
-    - `initial_pot_slice` (pte.CalibratedArray):
+    experimental_data : STEM4D
+        Experimental 4D-STEM data PyTree containing diffraction patterns,
+        scan positions, and calibration information.
+    initial_potential : CalibratedArray
         Initial guess for potential slice.
-    - `initial_beam` (pte.CalibratedArray):
+    initial_beam : CalibratedArray
         Initial guess for electron beam.
-    - `initial_pos_list` (Float[Array, "P 2"]):
-        Initial list of probe positions.
-    - `slice_thickness` (scalar_numeric):
+    slice_thickness : scalar_numeric
         Thickness of each slice.
-    - `voltage_kV` (scalar_numeric):
-        Accelerating voltage.
-    - `calib_ang` (scalar_float):
-        Calibration in angstroms.
-    - `save_every` (scalar_int):
-        Save every nth iteration.
-        Optional, default is 10.
-    - `num_iterations` (scalar_int):
-        Number of optimization iterations.
-        Optional, default is 1000.
-    - `learning_rate` (Optional[Union[scalar_float, Float[Array, "2"]]]):
+    save_every : scalar_int, optional
+        Save every nth iteration. Default is 10.
+    num_iterations : scalar_int, optional
+        Number of optimization iterations. Default is 1000.
+    learning_rate : scalar_float or Float[Array, "2"], optional
         Learning rate for potential slice and beam optimization.
         If the learning rate is a scalar, it is used for both
         potential slice and position optimization. If it is an array,
         the first element is used for potential slice and beam optimization,
         and the second element is used for position optimization.
-        Optional, default is 0.01.
-    - `loss_type` (str):
-        Type of loss function to use.
-        Optional, default is "mse".
-    - `optimizer_name` (str):
-        Name of optimizer to use.
-        Optional, default is "adam".
+        Default is 0.01.
+    loss_type : str, optional
+        Type of loss function to use. Default is "mse".
+    optimizer_name : str, optional
+        Name of optimizer to use. Default is "adam".
 
     Returns
     -------
-    - `final_potential` (pte.CalibratedArray):
-        Optimized potential slice.
-    - `final_beam` (pte.CalibratedArray):
-        Optimized electron beam.
-    - `pos_guess` (Float[Array, "P 2"]):
-        Optimized list of probe positions.
-    - `intermediate_potslices` (Complex[Array, "H W S"]):
-        Intermediate potential slices.
-    - `intermediate_beams` (Complex[Array, "H W S"]):
-        Intermediate electron beams.
-    - `intermediate_positions` (Float[Array, "P 2 S"]):
-        Intermediate probe positions.
+    tuple of (CalibratedArray, CalibratedArray, Float[Array, "P 2"], Complex[Array, "H W S"], Complex[Array, "H W S"], Float[Array, "P 2 S"])
+        - final_potential : CalibratedArray
+            Optimized potential slice.
+        - final_beam : CalibratedArray
+            Optimized electron beam.
+        - pos_guess : Float[Array, "P 2"]
+            Optimized list of probe positions.
+        - intermediate_potslices : Complex[Array, "H W S"]
+            Intermediate potential slices.
+        - intermediate_beams : Complex[Array, "H W S"]
+            Intermediate electron beams.
+        - intermediate_positions : Float[Array, "P 2 S"]
+            Intermediate probe positions.
     """
+    experimental_4dstem: Float[Array, "P H W"] = experimental_data.data
+    voltage_kV: Float[Array, " "] = experimental_data.voltage_kV
+    calib_ang: Float[Array, " "] = experimental_data.real_space_calib
+    initial_pos_list: Float[Array, "P 2"] = experimental_data.scan_positions
 
-    def forward_fn(
+    def _forward_fn(
         pot_slice: Complex[Array, "H W"],
         beam: Complex[Array, "H W"],
         pos_list: Float[Array, "P 2"],
     ) -> Float[Array, "P H W"]:
-        return stem_4D(
+        stem4d_result = stem_4d(
             pot_slice[None, ...],
             beam[None, ...],
             pos_list,
@@ -324,15 +301,18 @@ def single_slice_poscorrected(
             voltage_kV,
             calib_ang,
         )
+        return stem4d_result.data
 
-    loss_func = ptt.create_loss_function(forward_fn, experimental_4dstem, loss_type)
+    loss_func: Any = ptt.create_loss_function(
+        _forward_fn, experimental_4dstem, loss_type
+    )
 
     @jax.jit
-    def loss_and_grad(
+    def _loss_and_grad(
         pot_slice: Complex[Array, "H W"],
         beam: Complex[Array, "H W"],
         pos_list: Float[Array, "P 2"],
-    ) -> Tuple[Float[Array, ""], Dict[str, Array]]:
+    ) -> Tuple[Float[Array, " "], Dict[str, Array]]:
         loss, grads = jax.value_and_grad(loss_func, argnums=(0, 1, 2))(
             pot_slice, beam, pos_list
         )
@@ -343,13 +323,13 @@ def single_slice_poscorrected(
     beam_state: Any = optimizer.init(initial_beam.data_array.shape)
     pos_state: Any = optimizer.init(initial_pos_list.shape)
 
-    learning_rate: Float[Array, "..."] = jnp.array(learning_rate)
+    learning_rate: Float[Array, ...] = jnp.array(learning_rate)
 
     if len(learning_rate) == 1:
         learning_rate = jnp.array([learning_rate, learning_rate])
 
     @jax.jit
-    def update_step(
+    def _update_step(
         pot_slice: Complex[Array, "H W"],
         beam: Complex[Array, "H W"],
         pos_list: Float[Array, "P 2"],
@@ -363,11 +343,11 @@ def single_slice_poscorrected(
         Any,
         Any,
         Any,
-        Float[Array, ""],
+        Float[Array, " "],
     ]:
-        loss: Float[Array, ""]
+        loss: Float[Array, " "]
         grads: Dict[str, Array]
-        loss, grads = loss_and_grad(pot_slice, beam, pos_list)
+        loss, grads = _loss_and_grad(pot_slice, beam, pos_list)
         pot_slice, pot_slice_state = optimizer.update(
             pot_slice, grads["pot_slice"], pot_slice_state, learning_rate
         )
@@ -417,13 +397,13 @@ def single_slice_poscorrected(
             beam_state,
             pos_state,
             loss,
-        ) = update_step(
+        ) = _update_step(
             pot_guess, beam_guess, pos_guess, pot_slice_state, beam_state, pos_state
         )
 
         if ii % save_every == 0:
             print(f"Iteration {ii}, Loss: {loss}")
-            saver: scalar_int = jnp.floor(ii / save_every)
+            saver: Int[Array, ""] = jnp.floor(ii / save_every).astype(jnp.int32)
             intermediate_potslices = intermediate_potslices.at[:, :, saver].set(
                 pot_guess
             )
@@ -456,13 +436,10 @@ def single_slice_poscorrected(
 
 @jaxtyped(typechecker=typechecker)
 def single_slice_multi_modal(
-    experimental_4dstem: Float[Array, "P H W"],
+    experimental_data: STEM4D,
     initial_pot_slice: Complex[Array, "H W"],
     initial_beam: ProbeModes,
-    initial_pos_list: Float[Array, "P 2"],
     slice_thickness: scalar_numeric,
-    voltage_kV: scalar_numeric,
-    calib_ang: scalar_float,
     save_every: Optional[scalar_int] = 10,
     num_iterations: Optional[scalar_int] = 1000,
     learning_rate: Optional[Union[scalar_float, Float[Array, "2"]]] = 0.01,
@@ -475,67 +452,60 @@ def single_slice_multi_modal(
     Complex[Array, "H W S"],
     Complex[Array, "H W S"],
 ]:
-    """
-    Description
-    -----------
-    Create and run an optimization routine for 4D-STEM reconstruction with position correction.
+    """Create and run an optimization routine for 4D-STEM reconstruction with multi-modal probe.
 
     Parameters
     ----------
-    - `experimental_4dstem` (Float[Array, "P H W"]):
-        Experimental 4D-STEM data.
-    - `initial_pot_slice` (Complex[Array, "H W"]):
+    experimental_data : STEM4D
+        Experimental 4D-STEM data PyTree containing diffraction patterns,
+        scan positions, and calibration information.
+    initial_pot_slice : Complex[Array, "H W"]
         Initial guess for potential slice.
-    - `initial_beam` (Complex[Array, "H W"]):
-        Initial guess for electron beam.
-    - `initial_pos_list` (Float[Array, "P 2"]):
-        Initial list of probe positions.
-    - `slice_thickness` (scalar_numeric):
+    initial_beam : ProbeModes
+        Initial guess for electron beam with multiple probe modes.
+    slice_thickness : scalar_numeric
         Thickness of each slice.
-    - `voltage_kV` (scalar_numeric):
-        Accelerating voltage.
-    - `calib_ang` (scalar_float):
-        Calibration in angstroms.
-    - `save_every` (scalar_int):
-        Save every nth iteration.
-        Optional, default is 10.
-    - `num_iterations` (scalar_int):
-        Number of optimization iterations.
-        Optional, default is 1000.
-    - `learning_rate` (Optional[Union[scalar_float, Float[Array, "2"]]]):
+    save_every : scalar_int, optional
+        Save every nth iteration. Default is 10.
+    num_iterations : scalar_int, optional
+        Number of optimization iterations. Default is 1000.
+    learning_rate : scalar_float or Float[Array, "2"], optional
         Learning rate for potential slice and beam optimization.
         If the learning rate is a scalar, it is used for both
         potential slice and position optimization. If it is an array,
         the first element is used for potential slice and beam optimization,
         and the second element is used for position optimization.
-        Optional, default is 0.01.
-    - `loss_type` (str):
-        Type of loss function to use.
-        Optional, default is "mse".
-    - `optimizer_name` (str):
-        Name of optimizer to use.
-        Optional, default is "adam".
+        Default is 0.01.
+    loss_type : str, optional
+        Type of loss function to use. Default is "mse".
+    optimizer_name : str, optional
+        Name of optimizer to use. Default is "adam".
 
     Returns
     -------
-    - `pot_slice` (Complex[Array, "H W"]):
-        Optimized potential slice.
-    - `beam` (pte.ProbeModes):
-        Optimized electron beam.
-    - `pos_list` (Float[Array, "P 2"]):
-        Optimized list of probe positions.
-    - `intermediate_potslice` (Complex[Array, "H W S"]):
-        Intermediate potential slices.
-    - `intermediate_beam` (Complex[Array, "H W S"]):
-        Intermediate electron beams.
+    tuple of (Complex[Array, "H W"], ProbeModes, Float[Array, "P 2"], Complex[Array, "H W S"], Complex[Array, "H W S"])
+        - pot_slice : Complex[Array, "H W"]
+            Optimized potential slice.
+        - beam : ProbeModes
+            Optimized electron beam.
+        - pos_list : Float[Array, "P 2"]
+            Optimized list of probe positions.
+        - intermediate_potslice : Complex[Array, "H W S"]
+            Intermediate potential slices.
+        - intermediate_beam : Complex[Array, "H W S"]
+            Intermediate electron beams.
     """
+    experimental_4dstem: Float[Array, "P H W"] = experimental_data.data
+    voltage_kV: Float[Array, " "] = experimental_data.voltage_kV
+    calib_ang: Float[Array, " "] = experimental_data.real_space_calib
+    initial_pos_list: Float[Array, "P 2"] = experimental_data.scan_positions
 
-    def forward_fn(
+    def _forward_fn(
         pot_slice: Complex[Array, "H W"],
         beam: ProbeModes,
         pos_list: Float[Array, "P 2"],
     ) -> Float[Array, "P H W"]:
-        return stem_4D(
+        stem4d_result = stem_4d(
             pot_slice[None, ...],
             beam,
             pos_list,
@@ -543,15 +513,18 @@ def single_slice_multi_modal(
             voltage_kV,
             calib_ang,
         )
+        return stem4d_result.data
 
-    loss_func = ptt.create_loss_function(forward_fn, experimental_4dstem, loss_type)
+    loss_func: Any = ptt.create_loss_function(
+        _forward_fn, experimental_4dstem, loss_type
+    )
 
     @jax.jit
-    def loss_and_grad(
+    def _loss_and_grad(
         pot_slice: Complex[Array, "H W"],
         beam: ProbeModes,
         pos_list: Float[Array, "P 2"],
-    ) -> Tuple[Float[Array, ""], Dict[str, Any]]:
+    ) -> Tuple[Float[Array, " "], Dict[str, Any]]:
         loss, grads = jax.value_and_grad(loss_func, argnums=(0, 1, 2))(
             pot_slice, beam, pos_list
         )
@@ -562,12 +535,12 @@ def single_slice_multi_modal(
     beam_state: Any = optimizer.init(initial_beam.modes.shape)
     pos_state: Any = optimizer.init(initial_pos_list.shape)
 
-    learning_rate: Float[Array, "..."] = jnp.array(learning_rate)
+    learning_rate: Float[Array, ...] = jnp.array(learning_rate)
     if len(learning_rate.shape) == 0:
         learning_rate = jnp.array([learning_rate, learning_rate])
 
     @jax.jit
-    def update_step(
+    def _update_step(
         pot_slice: Complex[Array, "H W"],
         beam: ProbeModes,
         pos_list: Float[Array, "P 2"],
@@ -581,11 +554,11 @@ def single_slice_multi_modal(
         Any,
         Any,
         Any,
-        Float[Array, ""],
+        Float[Array, " "],
     ]:
-        loss: Float[Array, ""]
+        loss: Float[Array, " "]
         grads: Dict[str, Any]
-        loss, grads = loss_and_grad(pot_slice, beam, pos_list)
+        loss, grads = _loss_and_grad(pot_slice, beam, pos_list)
         pot_slice, pot_slice_state = optimizer.update(
             pot_slice, grads["pot_slice"], pot_slice_state, learning_rate[0]
         )
@@ -622,16 +595,16 @@ def single_slice_multi_modal(
     )
 
     for ii in range(num_iterations):
-        loss: Float[Array, ""]
+        loss: Float[Array, " "]
         pot_slice, beam, pos_list, pot_slice_state, beam_state, pos_state, loss = (
-            update_step(
+            _update_step(
                 pot_slice, beam, pos_list, pot_slice_state, beam_state, pos_state
             )
         )
 
         if ii % save_every == 0:
             print(f"Iteration {ii}, Loss: {loss}")
-            saver: scalar_int = jnp.floor(ii / save_every)
+            saver: Int[Array, ""] = jnp.floor(ii / save_every).astype(jnp.int32)
             intermediate_potslice = intermediate_potslice.at[:, :, saver].set(pot_slice)
             intermediate_beam = intermediate_beam.at[:, :, :, saver].set(beam.modes)
 
@@ -640,13 +613,10 @@ def single_slice_multi_modal(
 
 @jaxtyped(typechecker=typechecker)
 def multi_slice_multi_modal(
-    experimental_4dstem: Float[Array, "P H W"],
+    experimental_data: STEM4D,
     initial_pot_slice: Complex[Array, "H W"],
     initial_beam: Complex[Array, "H W"],
-    initial_pos_list: Float[Array, "P 2"],
     slice_thickness: scalar_numeric,
-    voltage_kV: scalar_numeric,
-    calib_ang: scalar_float,
     save_every: Optional[scalar_int] = 10,
     num_iterations: Optional[scalar_int] = 1000,
     learning_rate: Optional[scalar_float] = 0.001,
@@ -660,66 +630,57 @@ def multi_slice_multi_modal(
     Complex[Array, "H W S"],
     Complex[Array, "H W S"],
 ]:
-    """
-    Description
-    -----------
-    Create and run an optimization routine for 4D-STEM reconstruction with position correction.
+    """Create and run an optimization routine for multi-slice 4D-STEM reconstruction with position correction.
 
     Parameters
     ----------
-    - `experimental_4dstem` (Float[Array, "P H W"]):
-        Experimental 4D-STEM data.
-    - `initial_pot_slice` (Complex[Array, "H W"]):
+    experimental_data : STEM4D
+        Experimental 4D-STEM data PyTree containing diffraction patterns,
+        scan positions, and calibration information.
+    initial_pot_slice : Complex[Array, "H W"]
         Initial guess for potential slice.
-    - `initial_beam` (Complex[Array, "H W"]):
+    initial_beam : Complex[Array, "H W"]
         Initial guess for electron beam.
-    - `initial_pos_list` (Float[Array, "P 2"]):
-        Initial list of probe positions.
-    - `slice_thickness` (scalar_numeric):
+    slice_thickness : scalar_numeric
         Thickness of each slice.
-    - `voltage_kV` (scalar_numeric):
-        Accelerating voltage.
-    - `calib_ang` (scalar_float):
-        Calibration in angstroms.
-    - `save_every` (scalar_int):
-        Save every nth iteration.
-        Optional, default is 10.
-    - `num_iterations` (scalar_int):
-        Number of optimization iterations.
-        Optional, default is 1000.
-    - `learning_rate` (scalar_float):
-        Learning rate for potential slice and beam optimization.
-        Optional, default is 0.001.
-    - `pos_learning_rate` (scalar_float):
-        Learning rate for position optimization.
-        Optional, default is 0.01.
-    - `loss_type` (str):
-        Type of loss function to use.
-        Optional, default is "mse".
-    - `optimizer_name` (str):
-        Name of optimizer to use.
-        Optional, default is "adam".
+    save_every : scalar_int, optional
+        Save every nth iteration. Default is 10.
+    num_iterations : scalar_int, optional
+        Number of optimization iterations. Default is 1000.
+    learning_rate : scalar_float, optional
+        Learning rate for potential slice and beam optimization. Default is 0.001.
+    pos_learning_rate : scalar_float, optional
+        Learning rate for position optimization. Default is 0.01.
+    loss_type : str, optional
+        Type of loss function to use. Default is "mse".
+    optimizer_name : str, optional
+        Name of optimizer to use. Default is "adam".
 
     Returns
     -------
-    - `pot_slice` (Complex[Array, "H W"]):
-        Optimized potential slice.
-    - `beam` (Complex[Array, "H W"]):
-        Optimized electron beam.
-    - `pos_list` (Float[Array, "P 2"]):
-        Optimized list of probe positions.
-    - `intermediate_potslice` (Complex[Array, "H W S"]):
-        Intermediate potential slices.
-    - `intermediate_beam` (Complex[Array, "H W S"]):
-        Intermediate electron beams.
+    tuple of (Complex[Array, "H W"], Complex[Array, "H W"], Float[Array, "P 2"], Complex[Array, "H W S"], Complex[Array, "H W S"])
+        - pot_slice : Complex[Array, "H W"]
+            Optimized potential slice.
+        - beam : Complex[Array, "H W"]
+            Optimized electron beam.
+        - pos_list : Float[Array, "P 2"]
+            Optimized list of probe positions.
+        - intermediate_potslice : Complex[Array, "H W S"]
+            Intermediate potential slices.
+        - intermediate_beam : Complex[Array, "H W S"]
+            Intermediate electron beams.
     """
+    experimental_4dstem: Float[Array, "P H W"] = experimental_data.data
+    voltage_kV: Float[Array, " "] = experimental_data.voltage_kV
+    calib_ang: Float[Array, " "] = experimental_data.real_space_calib
+    initial_pos_list: Float[Array, "P 2"] = experimental_data.scan_positions
 
-    def forward_fn(
+    def _forward_fn(
         pot_slice: Complex[Array, "H W"],
         beam: Complex[Array, "H W"],
         pos_list: Float[Array, "P 2"],
     ) -> Float[Array, "P H W"]:
-        return stem_4D(
+        stem4d_result = stem_4d(
             pot_slice[None, ...],
             beam[None, ...],
             pos_list,
@@ -727,15 +688,18 @@ def multi_slice_multi_modal(
             voltage_kV,
             calib_ang,
         )
+        return stem4d_result.data
 
-    loss_func = ptt.create_loss_function(forward_fn, experimental_4dstem, loss_type)
+    loss_func: Any = ptt.create_loss_function(
+        _forward_fn, experimental_4dstem, loss_type
+    )
 
     @jax.jit
-    def loss_and_grad(
+    def _loss_and_grad(
         pot_slice: Complex[Array, "H W"],
         beam: Complex[Array, "H W"],
         pos_list: Float[Array, "P 2"],
-    ) -> Tuple[Float[Array, ""], Dict[str, Array]]:
+    ) -> Tuple[Float[Array, " "], Dict[str, Array]]:
         loss, grads = jax.value_and_grad(loss_func, argnums=(0, 1, 2))(
             pot_slice, beam, pos_list
         )
@@ -747,7 +711,7 @@ def multi_slice_multi_modal(
     pos_state: Any = optimizer.init(initial_pos_list.shape)
 
     @jax.jit
-    def update_step(
+    def _update_step(
         pot_slice: Complex[Array, "H W"],
         beam: Complex[Array, "H W"],
         pos_list: Float[Array, "P 2"],
@@ -761,11 +725,11 @@ def multi_slice_multi_modal(
         Any,
         Any,
         Any,
-        Float[Array, ""],
+        Float[Array, " "],
     ]:
-        loss: Float[Array, ""]
+        loss: Float[Array, " "]
         grads: Dict[str, Array]
-        loss, grads = loss_and_grad(pot_slice, beam, pos_list)
+        loss, grads = _loss_and_grad(pot_slice, beam, pos_list)
         pot_slice, pot_slice_state = optimizer.update(
             pot_slice, grads["pot_slice"], pot_slice_state, learning_rate
         )
@@ -799,16 +763,16 @@ def multi_slice_multi_modal(
     )
 
     for ii in range(num_iterations):
-        loss: Float[Array, ""]
+        loss: Float[Array, " "]
         pot_slice, beam, pos_list, pot_slice_state, beam_state, pos_state, loss = (
-            update_step(
+            _update_step(
                 pot_slice, beam, pos_list, pot_slice_state, beam_state, pos_state
             )
         )
 
         if ii % save_every == 0:
             print(f"Iteration {ii}, Loss: {loss}")
-            saver: scalar_int = jnp.floor(ii / save_every)
+            saver: Int[Array, ""] = jnp.floor(ii / save_every).astype(jnp.int32)
             intermediate_potslice = intermediate_potslice.at[:, :, saver].set(pot_slice)
             intermediate_beam = intermediate_beam.at[:, :, saver].set(beam)
 
