@@ -1,8 +1,7 @@
 """
-Module: photons.engine
-----------------------
-Extended Ptychographic Iterative Engine (ePIE) for optical ptychography.
-Local functions are prefixed with an underscore, and are not exported.
+Module: ptyrodactyl.photons.engine
+----------------------------------
+Engine for optical ptychography.
 
 Functions
 ---------
@@ -15,8 +14,8 @@ Functions
 - `single_pie_sequential`:
     Sequential processing over positions for refinement.
 
-Local Functions
----------------
+Internal Functions
+------------------
 - `_update_object_wavefront`:
     Updates object wavefront using rPIE algorithm
 - `_update_surface_pattern`:
@@ -33,9 +32,10 @@ Local Functions
 
 import jax
 import jax.numpy as jnp
-from beartype import beartype
 from beartype.typing import Callable, Optional, Tuple
-from jaxtyping import Array, Complex, Float, jaxtyped
+from jaxtyping import Array, Complex, Float
+
+from ptyrodactyl._decorators import beartype, jaxtyped
 
 from .lens_optics import angular_spectrum_prop
 from .photon_types import (MicroscopeData, OpticalWavefront, SampleFunction,
@@ -62,51 +62,51 @@ def epie_optical(
     gamma_surface: Optional[scalar_float] = 0.5,
     num_loops: Optional[scalar_integer] = 10,
 ) -> tuple[OpticalWavefront, SampleFunction]:
-    """
-    Description
-    -----------
-    Extended Ptychographic Iterative Engine for optical ptychography reconstruction.
+    """Extended Ptychographic Iterative Engine for optical ptychography reconstruction.
 
     Parameters
     ----------
-    - `microscope_data` (MicroscopeData):
-        Measured intensity data with positions
-    - `initial_object` (OpticalWavefront):
-        Initial guess for object wavefront
-    - `initial_surface` (SampleFunction):
-        Initial guess for surface pattern
-    - `pixel_mask` (Float[Array, "H W"]):
-        Pixel response mask for modeling sensor characteristics
-    - `propagation_distance_1` (scalar_float):
-        Distance from object to diffuser plane in meters
-    - `propagation_distance_2` (scalar_float):
-        Distance from diffuser to sensor plane in meters
-    - `magnification` (scalar_integer):
-        Magnification factor for downsampling
-    - `vmap_iterations` (scalar_integer):
-        Number of initial iterations to run in vmap mode for rapid convergence
-        If 0, use sequential mode for all iterations
-        If > 0, use vmap for first N iterations, then switch to sequential
-    - `alpha_object` (scalar_float):
-        Object update mixing parameter
-    - `gamma_object` (scalar_float):
-        Object update step size
-    - `alpha_surface` (scalar_float):
-        Surface update mixing parameter
-    - `gamma_surface` (scalar_float):
-        Surface update step size
-    - `num_loops` (scalar_integer):
-        Number of iteration loops
+    microscope_data : MicroscopeData
+        Measured intensity data with positions.
+    initial_object : OpticalWavefront
+        Initial guess for object wavefront.
+    initial_surface : SampleFunction
+        Initial guess for surface pattern.
+    pixel_mask : Float[Array, "H W"]
+        Pixel response mask for modeling sensor characteristics.
+    propagation_distance_1 : scalar_float
+        Distance from object to diffuser plane in meters.
+    propagation_distance_2 : scalar_float
+        Distance from diffuser to sensor plane in meters.
+    magnification : scalar_integer
+        Magnification factor for downsampling.
+    vmap_iterations : scalar_integer, optional
+        Number of initial iterations to run in vmap mode for rapid convergence.
+        If 0, use sequential mode for all iterations.
+        If > 0, use vmap for first N iterations, then switch to sequential.
+        Default is 0.
+    alpha_object : scalar_float, optional
+        Object update mixing parameter. Default is 0.1.
+    gamma_object : scalar_float, optional
+        Object update step size. Default is 0.5.
+    alpha_surface : scalar_float, optional
+        Surface update mixing parameter. Default is 0.1.
+    gamma_surface : scalar_float, optional
+        Surface update step size. Default is 0.5.
+    num_loops : scalar_integer, optional
+        Number of iteration loops. Default is 10.
 
     Returns
     -------
-    - `recovered_object` (OpticalWavefront):
-        Reconstructed object wavefront
-    - `recovered_surface` (SampleFunction):
-        Reconstructed surface pattern
+    tuple of (OpticalWavefront, SampleFunction)
+        - recovered_object : OpticalWavefront
+            Reconstructed object wavefront.
+        - recovered_surface : SampleFunction
+            Reconstructed surface pattern.
 
-    Flow
-    ----
+    Notes
+    -----
+    Algorithm:
     - Compute image data
     - Compute positions
     - Compute frequency grids
@@ -195,7 +195,7 @@ def single_pie_iteration(
     object_prop_ft: Complex[Array, "H W"],
     surface_pattern: Complex[Array, "H W"],
     measurement: Float[Array, "H W"],
-    position: Float[Array, "2"],
+    position: Float[Array, " 2"],
     frequency_x_grid: Float[Array, "H W"],
     frequency_y_grid: Float[Array, "H W"],
     pixel_mask: Float[Array, "H W"],
@@ -208,53 +208,52 @@ def single_pie_iteration(
     wavelength: scalar_float,
     dx: scalar_float,
 ) -> tuple[Complex[Array, "H W"], Complex[Array, "H W"]]:
-    """
-    Description
-    -----------
-    Single iteration of the extended PIE algorithm.
+    """Single iteration of the extended PIE algorithm.
 
     Parameters
     ----------
-    - `object_prop_ft` (Complex[Array, "H W"]):
-        Object wavefront at diffuser plane in Fourier domain
-    - `surface_pattern` (Complex[Array, "H W"]):
-        Surface pattern function
-    - `measurement` (Float[Array, "H W"]):
-        Measured intensity at current position
-    - `position` (Float[Array, "2"]):
-        Current scanning position [x, y]
-    - `frequency_x_grid` (Float[Array, "H W"]):
-        Frequency grid in x direction
-    - `frequency_y_grid` (Float[Array, "H W"]):
-        Frequency grid in y direction
-    - `pixel_mask` (Float[Array, "H W"]):
-        Pixel response mask for sensor modeling
-    - `propagation_distance_2` (scalar_float):
-        Distance from diffuser to sensor
-    - `magnification` (scalar_integer):
-        Downsampling magnification factor
-    - `alpha_object` (scalar_float):
-        Object update mixing parameter
-    - `gamma_object` (scalar_float):
-        Object update step size
-    - `alpha_surface` (scalar_float):
-        Surface update mixing parameter
-    - `gamma_surface` (scalar_float):
-        Surface update step size
-    - `wavelength` (scalar_float):
-        Wavelength of light in meters
-    - `dx` (scalar_float):
-        Pixel spacing in meters
+    object_prop_ft : Complex[Array, "H W"]
+        Object wavefront at diffuser plane in Fourier domain.
+    surface_pattern : Complex[Array, "H W"]
+        Surface pattern function.
+    measurement : Float[Array, "H W"]
+        Measured intensity at current position.
+    position : Float[Array, " 2"]
+        Current scanning position [x, y].
+    frequency_x_grid : Float[Array, "H W"]
+        Frequency grid in x direction.
+    frequency_y_grid : Float[Array, "H W"]
+        Frequency grid in y direction.
+    pixel_mask : Float[Array, "H W"]
+        Pixel response mask for sensor modeling.
+    propagation_distance_2 : scalar_float
+        Distance from diffuser to sensor.
+    magnification : scalar_integer
+        Downsampling magnification factor.
+    alpha_object : scalar_float
+        Object update mixing parameter.
+    gamma_object : scalar_float
+        Object update step size.
+    alpha_surface : scalar_float
+        Surface update mixing parameter.
+    gamma_surface : scalar_float
+        Surface update step size.
+    wavelength : scalar_float
+        Wavelength of light in meters.
+    dx : scalar_float
+        Pixel spacing in meters.
 
     Returns
     -------
-    - `updated_object_ft` (Complex[Array, "H W"]):
-        Updated object wavefront in Fourier domain
-    - `updated_surface` (Complex[Array, "H W"]):
-        Updated surface pattern
+    tuple of (Complex[Array, "H W"], Complex[Array, "H W"])
+        - updated_object_ft : Complex[Array, "H W"]
+            Updated object wavefront in Fourier domain.
+        - updated_surface : Complex[Array, "H W"]
+            Updated surface pattern.
 
-    Flow
-    ----
+    Notes
+    -----
+    Algorithm:
     - Compute object shifted
     - Compute surface plane
     - Compute surface propagation kernel
@@ -356,51 +355,49 @@ def single_pie_sequential(
     wavelength: scalar_float,
     dx: scalar_float,
 ) -> tuple[Complex[Array, "H W"], Complex[Array, "H W"]]:
-    """
-    Description
-    -----------
-    Sequential processing over positions using fori_loop for proper PIE convergence.
+    """Sequential processing over positions using fori_loop for proper PIE convergence.
 
     Parameters
     ----------
-    - `object_prop_ft` (Complex[Array, "H W"]):
-        Current object wavefront in Fourier domain
-    - `surface_pattern` (Complex[Array, "H W"]):
-        Current surface pattern
-    - `image_data` (Float[Array, "P H W"]):
-        Measurement data for all positions
-    - `positions` (Float[Array, "P 2"]):
-        Position coordinates for all measurements
-    - `frequency_x_grid` (Float[Array, "H W"]):
-        Frequency grid in x direction
-    - `frequency_y_grid` (Float[Array, "H W"]):
-        Frequency grid in y direction
-    - `pixel_mask` (Float[Array, "H W"]):
-        Pixel response mask for sensor modeling
-    - `propagation_distance_2` (scalar_float):
-        Distance from diffuser to sensor
-    - `magnification` (scalar_integer):
-        Downsampling magnification factor
-    - `alpha_object` (scalar_float):
-        Object update mixing parameter
-    - `gamma_object` (scalar_float):
-        Object update step size
-    - `alpha_surface` (scalar_float):
-        Surface update mixing parameter
-    - `gamma_surface` (scalar_float):
-        Surface update step size
-    - `wavelength` (scalar_float):
-        Wavelength of light in meters
-    - `dx` (scalar_float):
-        Pixel spacing in meters
+    object_prop_ft : Complex[Array, "H W"]
+        Current object wavefront in Fourier domain.
+    surface_pattern : Complex[Array, "H W"]
+        Current surface pattern.
+    image_data : Float[Array, "P H W"]
+        Measurement data for all positions.
+    positions : Float[Array, "P 2"]
+        Position coordinates for all measurements.
+    frequency_x_grid : Float[Array, "H W"]
+        Frequency grid in x direction.
+    frequency_y_grid : Float[Array, "H W"]
+        Frequency grid in y direction.
+    pixel_mask : Float[Array, "H W"]
+        Pixel response mask for sensor modeling.
+    propagation_distance_2 : scalar_float
+        Distance from diffuser to sensor.
+    magnification : scalar_integer
+        Downsampling magnification factor.
+    alpha_object : scalar_float
+        Object update mixing parameter.
+    gamma_object : scalar_float
+        Object update step size.
+    alpha_surface : scalar_float
+        Surface update mixing parameter.
+    gamma_surface : scalar_float
+        Surface update step size.
+    wavelength : scalar_float
+        Wavelength of light in meters.
+    dx : scalar_float
+        Pixel spacing in meters.
 
     Returns
     -------
-    - `updated_state` (tuple[Complex[Array, "H W"], Complex[Array, "H W"]]):
-        Updated object and surface state after sequential processing
+    tuple of (Complex[Array, "H W"], Complex[Array, "H W"])
+        Updated object and surface state after sequential processing.
 
-    Flow
-    ----
+    Notes
+    -----
+    Algorithm:
     - Compute number of positions
     - Define position body
     - Apply fori_loop over positions
@@ -456,31 +453,51 @@ def single_pie_vmap(
     wavelength: scalar_float,
     dx: scalar_float,
 ) -> tuple[Complex[Array, "H W"], Complex[Array, "H W"]]:
-    """
-    Description
-    -----------
-    Parallel processing over positions using vmap for faster but approximate PIE.
+    """Parallel processing over positions using vmap for faster but approximate PIE.
+
     All positions use the same initial state, then updates are averaged.
 
     Parameters
     ----------
-    - `object_prop_ft` (Complex[Array, "H W"]):
-        Current object wavefront in Fourier domain
-    - `surface_pattern` (Complex[Array, "H W"]):
-        Current surface pattern
-    - `image_data` (Float[Array, "P H W"]):
-        Measurement data for all positions
-    - `positions` (Float[Array, "P 2"]):
-        Position coordinates for all measurements
-    - Additional parameters for PIE algorithm
+    object_prop_ft : Complex[Array, "H W"]
+        Current object wavefront in Fourier domain.
+    surface_pattern : Complex[Array, "H W"]
+        Current surface pattern.
+    image_data : Float[Array, "P H W"]
+        Measurement data for all positions.
+    positions : Float[Array, "P 2"]
+        Position coordinates for all measurements.
+    frequency_x_grid : Float[Array, "H W"]
+        Frequency grid in x direction.
+    frequency_y_grid : Float[Array, "H W"]
+        Frequency grid in y direction.
+    pixel_mask : Float[Array, "H W"]
+        Pixel response mask for sensor modeling.
+    propagation_distance_2 : scalar_float
+        Distance from diffuser to sensor.
+    magnification : scalar_integer
+        Downsampling magnification factor.
+    alpha_object : scalar_float
+        Object update mixing parameter.
+    gamma_object : scalar_float
+        Object update step size.
+    alpha_surface : scalar_float
+        Surface update mixing parameter.
+    gamma_surface : scalar_float
+        Surface update step size.
+    wavelength : scalar_float
+        Wavelength of light in meters.
+    dx : scalar_float
+        Pixel spacing in meters.
 
     Returns
     -------
-    - `updated_state` (tuple[Complex[Array, "H W"], Complex[Array, "H W"]]):
-        Updated object and surface state after parallel processing and averaging
+    tuple of (Complex[Array, "H W"], Complex[Array, "H W"])
+        Updated object and surface state after parallel processing and averaging.
 
-    Flow
-    ----
+    Notes
+    -----
+    Algorithm:
     - Apply vmap over all positions using same initial state
     - Compute average of all object updates
     - Compute average of all surface updates
@@ -542,33 +559,31 @@ def _update_object_wavefront(
     alpha_object: scalar_float,
     gamma_object: scalar_float,
 ) -> Complex[Array, "H W"]:
-    """
-    Description
-    -----------
-    Updates object wavefront using rPIE algorithm.
+    """Update object wavefront using rPIE algorithm.
 
     Parameters
     ----------
-    - `object_shift` (Complex[Array, "H W"]):
-        Current shifted object wavefront
-    - `surface_pattern` (Complex[Array, "H W"]):
-        Surface pattern function
-    - `surface_plane` (Complex[Array, "H W"]):
-        Current exit wave at surface
-    - `surface_plane_new` (Complex[Array, "H W"]):
-        Updated exit wave at surface
-    - `alpha_object` (scalar_float):
-        Mixing parameter for denominator
-    - `gamma_object` (scalar_float):
-        Update step size
+    object_shift : Complex[Array, "H W"]
+        Current shifted object wavefront.
+    surface_pattern : Complex[Array, "H W"]
+        Surface pattern function.
+    surface_plane : Complex[Array, "H W"]
+        Current exit wave at surface.
+    surface_plane_new : Complex[Array, "H W"]
+        Updated exit wave at surface.
+    alpha_object : scalar_float
+        Mixing parameter for denominator.
+    gamma_object : scalar_float
+        Update step size.
 
     Returns
     -------
-    - `updated_object` (Complex[Array, "H W"]):
-        Updated object wavefront
+    Complex[Array, "H W"]
+        Updated object wavefront.
 
-    Flow
-    ----
+    Notes
+    -----
+    Algorithm:
     - Compute surface conjugate
     - Compute difference between current and updated surface plane
     - Compute surface absolute squared
@@ -600,37 +615,31 @@ def _update_surface_pattern(
     alpha_surface: scalar_float,
     gamma_surface: scalar_float,
 ) -> Complex[Array, "H W"]:
-    """
-    Description
-    -----------
-    Updates surface pattern using modified PIE algorithm.
+    """Update surface pattern using modified PIE algorithm.
 
     Parameters
     ----------
-    - `surface_pattern` (Complex[Array, "H W"]):
-        Current surface pattern
-    - `object_shift` (Complex[Array, "H W"]):
-        Shifted object wavefront
-    - `surface_plane` (Complex[Array, "H W"]):
-        Current exit wave at surface
-    - `surface_plane_new` (Complex[Array, "H W"]):
-        Updated exit wave at surface
-    - `alpha_surface` (scalar_float):
-        Mixing parameter for denominator
-    - `gamma_surface` (scalar_float):
-        Update step size
-    - `wavelength` (scalar_float):
-        Wavelength of light in meters
-    - `dx` (scalar_float):
-        Pixel spacing in meters
+    surface_pattern : Complex[Array, "H W"]
+        Current surface pattern.
+    object_shift : Complex[Array, "H W"]
+        Shifted object wavefront.
+    surface_plane : Complex[Array, "H W"]
+        Current exit wave at surface.
+    surface_plane_new : Complex[Array, "H W"]
+        Updated exit wave at surface.
+    alpha_surface : scalar_float
+        Mixing parameter for denominator.
+    gamma_surface : scalar_float
+        Update step size.
 
     Returns
     -------
-    - `updated_surface` (Complex[Array, "H W"]):
-        Updated surface pattern
+    Complex[Array, "H W"]
+        Updated surface pattern.
 
-    Flow
-    ----
+    Notes
+    -----
+    Algorithm:
     - Compute object conjugate
     - Compute difference between current and updated surface plane
     - Compute object absolute squared
@@ -657,23 +666,21 @@ def _update_surface_pattern(
 def _apply_coherent_transfer_function(
     field_ft: Complex[Array, "H W"],
 ) -> Complex[Array, "H W"]:
-    """
-    Description
-    -----------
-    Applies coherent transfer function in Fourier domain.
+    """Apply coherent transfer function in Fourier domain.
 
     Parameters
     ----------
-    - `field_ft` (Complex[Array, "H W"]):
-        Field in Fourier domain
+    field_ft : Complex[Array, "H W"]
+        Field in Fourier domain.
 
     Returns
     -------
-    - `result` (Complex[Array, "H W"]):
-        Field after CTF application
+    Complex[Array, "H W"]
+        Field after CTF application.
 
-    Flow
-    ----
+    Notes
+    -----
+    Algorithm:
     - Compute CTF
     - Apply CTF to field
     - Return result
@@ -686,34 +693,33 @@ def _apply_coherent_transfer_function(
 @jaxtyped(typechecker=beartype)
 def _apply_position_shift(
     field_ft: Complex[Array, "H W"],
-    position: Float[Array, "2"],
+    position: Float[Array, " 2"],
     frequency_x_grid: Float[Array, "H W"],
     frequency_y_grid: Float[Array, "H W"],
 ) -> Complex[Array, "H W"]:
-    """
-    Description
-    -----------
-    Applies position shift using phase multiplication in Fourier domain.
+    """Apply position shift using phase multiplication in Fourier domain.
+
     Implements MATLAB: Hs = exp(-1j*2*pi.*(FX.*xlocation(tt)/imSize+FY.*ylocation(tt)/imSize))
 
     Parameters
     ----------
-    - `field_ft` (Complex[Array, "H W"]):
-        Field in Fourier domain
-    - `position` (Float[Array, "2"]):
-        Position shift [x, y]
-    - `frequency_x_grid` (Float[Array, "H W"]):
-        Frequency grid in x direction
-    - `frequency_y_grid` (Float[Array, "H W"]):
-        Frequency grid in y direction
+    field_ft : Complex[Array, "H W"]
+        Field in Fourier domain.
+    position : Float[Array, " 2"]
+        Position shift [x, y].
+    frequency_x_grid : Float[Array, "H W"]
+        Frequency grid in x direction.
+    frequency_y_grid : Float[Array, "H W"]
+        Frequency grid in y direction.
 
     Returns
     -------
-    - `shifted_field` (Complex[Array, "H W"]):
-        Position-shifted field in real space
+    Complex[Array, "H W"]
+        Position-shifted field in real space.
 
-    Flow
-    ----
+    Notes
+    -----
+    Algorithm:
     - Compute image size
     - Compute phase factor
     - Compute position-shifted field in Fourier domain
@@ -744,29 +750,27 @@ def _get_propagation_kernel(
     wavelength: scalar_float,
     dx: scalar_float,
 ) -> Complex[Array, "H W"]:
-    """
-    Description
-    -----------
-    Returns propagation kernel H_d for free space propagation.
+    """Return propagation kernel H_d for free space propagation.
 
     Parameters
     ----------
-    - `field_shape` (Tuple[int, int]):
-        Shape of the field (height, width)
-    - `distance` (scalar_float):
-        Propagation distance in meters
-    - `wavelength` (scalar_float):
-        Wavelength in meters
-    - `dx` (scalar_float):
-        Pixel spacing in meters
+    field_shape : Tuple[int, int]
+        Shape of the field (height, width).
+    distance : scalar_float
+        Propagation distance in meters.
+    wavelength : scalar_float
+        Wavelength in meters.
+    dx : scalar_float
+        Pixel spacing in meters.
 
     Returns
     -------
-    - `kernel` (Complex[Array, "H W"]):
-        Propagation kernel for angular spectrum method
+    Complex[Array, "H W"]
+        Propagation kernel for angular spectrum method.
 
-    Flow
-    ----
+    Notes
+    -----
+    Algorithm:
     - Compute height and width of field
     - Compute frequency grid in x direction
     - Compute frequency grid in y direction
@@ -780,8 +784,8 @@ def _get_propagation_kernel(
     height: int
     width: int
     height, width = field_shape
-    frequency_x: Float[Array, "W"] = jnp.fft.fftfreq(width, dx)
-    frequency_y: Float[Array, "H"] = jnp.fft.fftfreq(height, dx)
+    frequency_x: Float[Array, " W"] = jnp.fft.fftfreq(width, dx)
+    frequency_y: Float[Array, " H"] = jnp.fft.fftfreq(height, dx)
     frequency_x_grid: Float[Array, "H W"]
     frequency_y_grid: Float[Array, "H W"]
     frequency_x_grid, frequency_y_grid = jnp.meshgrid(frequency_x, frequency_y)
@@ -800,28 +804,27 @@ def _compute_sensor_intensity(
     pixel_mask: Float[Array, "H W"],
     magnification: scalar_integer,
 ) -> Float[Array, "H W"]:
-    """
-    Description
-    -----------
-    Computes sensor plane intensity with pixel response mask and downsampling.
+    """Compute sensor plane intensity with pixel response mask and downsampling.
+
     Implements the MATLAB: conv2(pixelMask.*abs(sensorPlane).^2, ones(mag,mag))
 
     Parameters
     ----------
-    - `sensor_field` (Complex[Array, "H W"]):
-        Complex field at sensor plane
-    - `pixel_mask` (Float[Array, "H W"]):
-        Pixel response mask modeling sensor characteristics
-    - `magnification` (scalar_integer):
-        Downsampling magnification factor
+    sensor_field : Complex[Array, "H W"]
+        Complex field at sensor plane.
+    pixel_mask : Float[Array, "H W"]
+        Pixel response mask modeling sensor characteristics.
+    magnification : scalar_integer
+        Downsampling magnification factor.
 
     Returns
     -------
-    - `downsampled_intensity` (Float[Array, "H W"]):
-        Downsampled intensity pattern
+    Float[Array, "H W"]
+        Downsampled intensity pattern.
 
-    Flow
-    ----
+    Notes
+    -----
+    Algorithm:
     - Compute intensity
     - Apply pixel mask
     - Compute kernel
@@ -844,27 +847,26 @@ def _compute_sensor_intensity(
 def _create_frequency_grids(
     field: Complex[Array, "H W"], dx: scalar_float
 ) -> Tuple[Float[Array, "H W"], Float[Array, "H W"]]:
-    """
-    Description
-    -----------
-    Creates frequency grids for Fourier transforms.
+    """Create frequency grids for Fourier transforms.
 
     Parameters
     ----------
-    - `field` (Complex[Array, "H W"]):
-        Input field to determine grid size
-    - `dx` (scalar_float):
-        Spatial sampling interval
+    field : Complex[Array, "H W"]
+        Input field to determine grid size.
+    dx : scalar_float
+        Spatial sampling interval.
 
     Returns
     -------
-    - `frequency_x_grid` (Float[Array, "H W"]):
-        Frequency grid in x direction
-    - `frequency_y_grid` (Float[Array, "H W"]):
-        Frequency grid in y direction
+    tuple of (Float[Array, "H W"], Float[Array, "H W"])
+        - frequency_x_grid : Float[Array, "H W"]
+            Frequency grid in x direction.
+        - frequency_y_grid : Float[Array, "H W"]
+            Frequency grid in y direction.
 
-    Flow
-    ----
+    Notes
+    -----
+    Algorithm:
     - Compute height and width of field
     - Compute frequency grid in x direction
     - Compute frequency grid in y direction
@@ -873,8 +875,8 @@ def _create_frequency_grids(
     """
     height: scalar_integer = field.shape[0]
     width: scalar_integer = field.shape[1]
-    frequency_x: Float[Array, "W"] = jnp.fft.fftfreq(width, dx)
-    frequency_y: Float[Array, "H"] = jnp.fft.fftfreq(height, dx)
+    frequency_x: Float[Array, " W"] = jnp.fft.fftfreq(width, dx)
+    frequency_y: Float[Array, " H"] = jnp.fft.fftfreq(height, dx)
     frequency_x_grid: Float[Array, "H W"]
     frequency_y_grid: Float[Array, "H W"]
     frequency_x_grid, frequency_y_grid = jnp.meshgrid(frequency_x, frequency_y)
@@ -885,21 +887,17 @@ def _create_frequency_grids(
 def _get_ctf(
     field_shape: Optional[Tuple[int, int]] = (256, 256)
 ) -> Complex[Array, "H W"]:
-    """
-    Description
-    -----------
-    Returns a placeholder coherent transfer function.
+    """Return a placeholder coherent transfer function.
 
     Parameters
     ----------
-    - `field_shape` (Optional[Tuple[int, int]]):
-        Shape of the field (height, width)
-        Default is (256, 256)
+    field_shape : Tuple[int, int], optional
+        Shape of the field (height, width). Default is (256, 256).
 
     Returns
     -------
-    - `ctf` (Complex[Array, "H W"]):
-        Coherent transfer function
+    Complex[Array, "H W"]
+        Coherent transfer function.
     """
     ctf: Complex[Array, "H W"] = jnp.ones(field_shape, dtype=complex)
     return ctf
