@@ -1,18 +1,18 @@
 """
-Module: photons.photon_types
-----------------------------
+Module: ptyrodactyl.photons.photon_types
+----------------------------------------
 Data structures and type definitions for optical ptychography.
 
 Type Aliases
 ------------
 - `scalar_float`:
-    A type alias for float or Float[Array, ""]
+    A type alias for float or Float[Array, " "]
 - `scalar_integer`:
-    A type alias for int or Int[Array, ""]
+    A type alias for int or Int[Array, " "]
 - `scalar_complex`:
-    A type alias for complex or Complex[Array, ""]
+    A type alias for complex or Complex[Array, " "]
 - `scalar_numeric`:
-    A type alias for int, float, complex or Num[Array, ""]
+    A type alias for int, float, complex or Num[Array, " "]
 - `non_jax_number`:
     A type alias for int, float or complex
 
@@ -32,7 +32,7 @@ Classes
     A named tuple for storing a single diffraction pattern
 
 Factory Functions
-----------------
+-----------------
 - `make_lens_params`:
     Creates a LensParams instance with runtime type checking
 - `make_grid_params`:
@@ -46,48 +46,48 @@ Factory Functions
 - `make_sample_function`:
     Creates a SampleFunction instance with runtime type checking
 
-    Note: Always use these factory functions instead of directly instantiating the
-    NamedTuple classes to ensure proper runtime type checking of the contents.
+Notes
+-----
+Always use these factory functions instead of directly instantiating the
+NamedTuple classes to ensure proper runtime type checking of the contents.
 """
 
 import jax
 import jax.numpy as jnp
-from beartype import beartype
-from beartype.typing import NamedTuple, TypeAlias, Union
+from beartype.typing import NamedTuple, Tuple, TypeAlias, Union
 from jax import lax
 from jax.tree_util import register_pytree_node_class
-from jaxtyping import Array, Complex, Float, Int, Num, jaxtyped
+from jaxtyping import Array, Complex, Float, Int, Num
+
+from ptyrodactyl._decorators import beartype, jaxtyped
 
 jax.config.update("jax_enable_x64", True)
 
-scalar_float: TypeAlias = Union[float, Float[Array, ""]]
-scalar_integer: TypeAlias = Union[int, Int[Array, ""]]
-scalar_complex: TypeAlias = Union[complex, Complex[Array, ""]]
-scalar_numeric: TypeAlias = Union[int, float, complex, Num[Array, ""]]
+scalar_float: TypeAlias = Union[float, Float[Array, " "]]
+scalar_integer: TypeAlias = Union[int, Int[Array, " "]]
+scalar_complex: TypeAlias = Union[complex, Complex[Array, " "]]
+scalar_numeric: TypeAlias = Union[int, float, complex, Num[Array, " "]]
 non_jax_number: TypeAlias = Union[int, float, complex]
 
 
 @jaxtyped(typechecker=beartype)
 @register_pytree_node_class
 class LensParams(NamedTuple):
-    """
-    Description
-    -----------
-    PyTree structure for lens parameters
+    """PyTree structure for lens parameters.
 
     Attributes
     ----------
-    - `focal_length` (scalar_float):
+    focal_length : scalar_float
         Focal length of the lens in meters
-    - `diameter` (scalar_float):
+    diameter : scalar_float
         Diameter of the lens in meters
-    - `n` (scalar_float):
+    n : scalar_float
         Refractive index of the lens material
-    - `center_thickness` (scalar_float):
+    center_thickness : scalar_float
         Thickness at the center of the lens in meters
-    - `r1` (scalar_float):
+    r1 : scalar_float
         Radius of curvature of the first surface in meters (positive for convex)
-    - `r2` (scalar_float):
+    r2 : scalar_float
         Radius of curvature of the second surface in meters (positive for convex)
 
     Notes
@@ -125,20 +125,17 @@ class LensParams(NamedTuple):
 @jaxtyped(typechecker=beartype)
 @register_pytree_node_class
 class GridParams(NamedTuple):
-    """
-    Description
-    -----------
-    PyTree structure for computational grid parameters
+    """PyTree structure for computational grid parameters.
 
     Attributes
     ----------
-    - `xx` (Float[Array, "H W"]):
+    xx : Float[Array, "H W"]
         Spatial grid in the x-direction
-    - `yy` (Float[Array, "H W"]):
+    yy : Float[Array, "H W"]
         Spatial grid in the y-direction
-    - `phase_profile` (Float[Array, "H W"]):
+    phase_profile : Float[Array, "H W"]
         Phase profile of the optical field
-    - `transmission` (Float[Array, "H W"]):
+    transmission : Float[Array, "H W"]
         Transmission profile of the optical field
 
     Notes
@@ -173,20 +170,17 @@ class GridParams(NamedTuple):
 @jaxtyped(typechecker=beartype)
 @register_pytree_node_class
 class OpticalWavefront(NamedTuple):
-    """
-    Description
-    -----------
-    PyTree structure for representing an optical wavefront.
+    """PyTree structure for representing an optical wavefront.
 
     Attributes
     ----------
-    - `field` (Complex[Array, "H W"]):
+    field : Complex[Array, "H W"]
         Complex amplitude of the optical field.
-    - `wavelength` (scalar_float):
+    wavelength : scalar_float
         Wavelength of the optical wavefront in meters.
-    - `dx` (scalar_float):
+    dx : scalar_float
         Spatial sampling interval (grid spacing) in meters.
-    - `z_position` (scalar_float):
+    z_position : scalar_float
         Axial position of the wavefront along the propagation direction in meters.
     """
 
@@ -214,25 +208,22 @@ class OpticalWavefront(NamedTuple):
 @jaxtyped(typechecker=beartype)
 @register_pytree_node_class
 class MicroscopeData(NamedTuple):
-    """
-    Description
-    -----------
-    PyTree structure for representing an 3D or 4D microscope image.
+    """PyTree structure for representing an 3D or 4D microscope image.
 
     Attributes
     ----------
-    - `image_data` (Float[Array, "P H W"] | Float[Array, "X Y H W"]):
+    image_data : Float[Array, "P H W"] | Float[Array, "X Y H W"]
         3D or 4D image data representing the optical field.
-    - `positions` (Num[Array, "P 2"]):
+    positions : Num[Array, " P 2"]
         Positions of the images during collection.
-    - `wavelength` (scalar_float):
+    wavelength : scalar_float
         Wavelength of the optical wavefront in meters.
-    - `dx` (scalar_float):
+    dx : scalar_float
         Spatial sampling interval (grid spacing) in meters.
     """
 
     image_data: Union[Float[Array, "P H W"], Float[Array, "X Y H W"]]
-    positions: Num[Array, "P 2"]
+    positions: Num[Array, " P 2"]
     wavelength: scalar_float
     dx: scalar_float
 
@@ -255,16 +246,13 @@ class MicroscopeData(NamedTuple):
 @jaxtyped(typechecker=beartype)
 @register_pytree_node_class
 class SampleFunction(NamedTuple):
-    """
-    Description
-    -----------
-    PyTree structure for representing an 3D or 4D microscope image.
+    """PyTree structure for representing a sample function.
 
     Attributes
     ----------
-    - `sample` (Complex[Array, "H W"]):
+    sample : Complex[Array, "H W"]
         The sample function.
-    - `dx` (scalar_float):
+    dx : scalar_float
         Spatial sampling interval (grid spacing) in meters.
     """
 
@@ -288,18 +276,15 @@ class SampleFunction(NamedTuple):
 @jaxtyped(typechecker=beartype)
 @register_pytree_node_class
 class Diffractogram(NamedTuple):
-    """
-    Description
-    -----------
-    PyTree structure for representing a single diffractogram.
+    """PyTree structure for representing a single diffractogram.
 
     Attributes
     ----------
-    - `image` (Float[Array, "H W"]):
+    image : Float[Array, "H W"]
         Image data.
-    - `wavelength` (scalar_float):
+    wavelength : scalar_float
         Wavelength of the optical wavefront in meters.
-    - `dx` (scalar_float):
+    dx : scalar_float
         Spatial sampling interval (grid spacing) in meters.
     """
 
@@ -331,38 +316,37 @@ def make_lens_params(
     r1: scalar_float,
     r2: scalar_float,
 ) -> LensParams:
-    """
-    Description
-    -----------
-    JAX-safe factory function for LensParams with data validation.
+    """JAX-safe factory function for LensParams with data validation.
 
     Parameters
     ----------
-    - `focal_length` (scalar_float):
+    focal_length : scalar_float
         Focal length of the lens in meters
-    - `diameter` (scalar_float):
+    diameter : scalar_float
         Diameter of the lens in meters
-    - `n` (scalar_float):
+    n : scalar_float
         Refractive index of the lens material
-    - `center_thickness` (scalar_float):
+    center_thickness : scalar_float
         Thickness at the center of the lens in meters
-    - `r1` (scalar_float):
+    r1 : scalar_float
         Radius of curvature of the first surface in meters (positive for convex)
-    - `r2` (scalar_float):
+    r2 : scalar_float
         Radius of curvature of the second surface in meters (positive for convex)
 
     Returns
     -------
-    - `lens_params` (LensParams):
+    LensParams
         Validated lens parameters instance
 
     Raises
     ------
-    - ValueError:
+    ValueError
         If parameters are invalid or out of valid ranges
 
-    Flow
-    ----
+    Notes
+    -----
+    Algorithm:
+
     - Convert inputs to JAX arrays
     - Validate parameters:
         - Check focal_length is positive
@@ -448,34 +432,33 @@ def make_grid_params(
     phase_profile: Float[Array, "H W"],
     transmission: Float[Array, "H W"],
 ) -> GridParams:
-    """
-    Description
-    -----------
-    JAX-safe factory function for GridParams with data validation.
+    """JAX-safe factory function for GridParams with data validation.
 
     Parameters
     ----------
-    - `xx` (Float[Array, "H W"]):
+    xx : Float[Array, "H W"]
         Spatial grid in the x-direction
-    - `yy` (Float[Array, "H W"]):
+    yy : Float[Array, "H W"]
         Spatial grid in the y-direction
-    - `phase_profile` (Float[Array, "H W"]):
+    phase_profile : Float[Array, "H W"]
         Phase profile of the optical field
-    - `transmission` (Float[Array, "H W"]):
+    transmission : Float[Array, "H W"]
         Transmission profile of the optical field
 
     Returns
     -------
-    - `grid_params` (GridParams):
+    GridParams
         Validated grid parameters instance
 
     Raises
     ------
-    - ValueError:
+    ValueError
         If array shapes are inconsistent or data is invalid
 
-    Flow
-    ----
+    Notes
+    -----
+    Algorithm:
+
     - Convert inputs to JAX arrays
     - Validate array shapes:
         - Check all arrays are 2D
@@ -573,39 +556,38 @@ def make_grid_params(
 
 @jaxtyped(typechecker=beartype)
 def make_optical_wavefront(
-    field: Complex[Array, "H W"],
+    field: Complex[Array, " hh ww"],
     wavelength: scalar_float,
     dx: scalar_float,
     z_position: scalar_float,
 ) -> OpticalWavefront:
-    """
-    Description
-    -----------
-    JAX-safe factory function for OpticalWavefront with data validation.
+    """JAX-safe factory function for OpticalWavefront with data validation.
 
     Parameters
     ----------
-    - `field` (Complex[Array, "H W"]):
+    field : Complex[Array, " hh ww"]
         Complex amplitude of the optical field
-    - `wavelength` (scalar_float):
+    wavelength : scalar_float
         Wavelength of the optical wavefront in meters
-    - `dx` (scalar_float):
+    dx : scalar_float
         Spatial sampling interval (grid spacing) in meters
-    - `z_position` (scalar_float):
+    z_position : scalar_float
         Axial position of the wavefront along the propagation direction in meters
 
     Returns
     -------
-    - `wavefront` (OpticalWavefront):
+    OpticalWavefront
         Validated optical wavefront instance
 
     Raises
     ------
-    - ValueError:
+    ValueError
         If data is invalid or parameters are out of valid ranges
 
-    Flow
-    ----
+    Notes
+    -----
+    Algorithm:
+
     - Convert inputs to JAX arrays
     - Validate field array:
         - Check it's 2D
@@ -616,10 +598,10 @@ def make_optical_wavefront(
         - Check z_position is finite
     - Create and return OpticalWavefront instance
     """
-    field = jnp.asarray(field, dtype=jnp.complex128)
-    wavelength = jnp.asarray(wavelength, dtype=jnp.float64)
-    dx = jnp.asarray(dx, dtype=jnp.float64)
-    z_position = jnp.asarray(z_position, dtype=jnp.float64)
+    field: Complex[Array, " hh ww"] = jnp.asarray(field, dtype=jnp.complex128)
+    wavelength: Float[Array, " "] = jnp.asarray(wavelength, dtype=jnp.float64)
+    dx: Float[Array, " "] = jnp.asarray(dx, dtype=jnp.float64)
+    z_position: Float[Array, " "] = jnp.asarray(z_position, dtype=jnp.float64)
 
     def validate_and_create():
         def check_2d_field():
@@ -683,45 +665,44 @@ def make_optical_wavefront(
 
 @jaxtyped(typechecker=beartype)
 def make_microscope_data(
-    image_data: Union[Float[Array, "P H W"], Float[Array, "X Y H W"]],
-    positions: Num[Array, "P 2"],
+    image_data: Union[Float[Array, " pp hh ww"], Float[Array, " xx yy hh ww"]],
+    positions: Num[Array, " pp 2"],
     wavelength: scalar_float,
     dx: scalar_float,
 ) -> MicroscopeData:
-    """
-    Description
-    -----------
-    JAX-safe factory function for MicroscopeData with data validation.
+    """JAX-safe factory function for MicroscopeData with data validation.
 
     Parameters
     ----------
-    - `image_data` (Union[Float[Array, "P H W"], Float[Array, "X Y H W"]]):
+    image_data : Union[Float[Array, " pp hh ww"], Float[Array, " xx yy hh ww"]]
         3D or 4D image data representing the optical field
-    - `positions` (Num[Array, "P 2"]):
+    positions : Num[Array, " pp 2"]
         Positions of the images during collection
-    - `wavelength` (scalar_float):
+    wavelength : scalar_float
         Wavelength of the optical wavefront in meters
-    - `dx` (scalar_float):
+    dx : scalar_float
         Spatial sampling interval (grid spacing) in meters
 
     Returns
     -------
-    - `microscope_data` (MicroscopeData):
+    MicroscopeData
         Validated microscope data instance
 
     Raises
     ------
-    - ValueError:
+    ValueError
         If data is invalid or parameters are out of valid ranges
 
-    Flow
-    ----
+    Notes
+    -----
+    Algorithm:
+
     - Convert inputs to JAX arrays
     - Validate image_data:
         - Check it's 3D or 4D
         - Ensure all values are finite and non-negative
     - Validate positions:
-        - Check it's 2D with shape (P, 2)
+        - Check it's 2D with shape (pp, 2)
         - Ensure all values are finite
     - Validate parameters:
         - Check wavelength is positive
@@ -730,22 +711,34 @@ def make_microscope_data(
         - Check P matches between image_data and positions
     - Create and return MicroscopeData instance
     """
-    image_data = jnp.asarray(image_data, dtype=jnp.float64)
-    positions = jnp.asarray(positions, dtype=jnp.float64)
-    wavelength = jnp.asarray(wavelength, dtype=jnp.float64)
-    dx = jnp.asarray(dx, dtype=jnp.float64)
+    image_data: Union[Float[Array, " pp hh ww"], Float[Array, " xx yy hh ww"]] = (
+        jnp.asarray(image_data, dtype=jnp.float64)
+    )
+    positions: Num[Array, " pp 2"] = jnp.asarray(positions, dtype=jnp.float64)
+    wavelength: Float[Array, " "] = jnp.asarray(wavelength, dtype=jnp.float64)
+    dx: Float[Array, " "] = jnp.asarray(dx, dtype=jnp.float64)
+    expected_image_dim = 2
+    expected_diffractogram_dim_3d: int = 3
+    expected_diffractogram_dim_4d: int = 4
 
-    def validate_and_create():
-        def check_image_dimensions():
+    def validate_and_create() -> MicroscopeData:
+        def check_image_dimensions() -> (
+            Union[Float[Array, "P H W"], Float[Array, "X Y H W"]]
+        ):
             return lax.cond(
-                jnp.logical_or(image_data.ndim == 3, image_data.ndim == 4),
+                jnp.logical_or(
+                    image_data.ndim == expected_diffractogram_dim_3d,
+                    image_data.ndim == expected_diffractogram_dim_4d,
+                ),
                 lambda: image_data,
                 lambda: lax.stop_gradient(
                     lax.cond(False, lambda: image_data, lambda: image_data)
                 ),
             )
 
-        def check_image_finite():
+        def check_image_finite() -> (
+            Union[Float[Array, "P H W"], Float[Array, "X Y H W"]]
+        ):
             return lax.cond(
                 jnp.all(jnp.isfinite(image_data)),
                 lambda: image_data,
@@ -754,7 +747,9 @@ def make_microscope_data(
                 ),
             )
 
-        def check_image_nonnegative():
+        def check_image_nonnegative() -> (
+            Union[Float[Array, "P H W"], Float[Array, "X Y H W"]]
+        ):
             return lax.cond(
                 jnp.all(image_data >= 0),
                 lambda: image_data,
@@ -763,16 +758,16 @@ def make_microscope_data(
                 ),
             )
 
-        def check_positions_shape():
+        def check_positions_shape() -> Num[Array, " P 2"]:
             return lax.cond(
-                positions.shape[1] == 2,
+                positions.shape[1] == expected_image_dim,
                 lambda: positions,
                 lambda: lax.stop_gradient(
                     lax.cond(False, lambda: positions, lambda: positions)
                 ),
             )
 
-        def check_positions_finite():
+        def check_positions_finite() -> Num[Array, " P 2"]:
             return lax.cond(
                 jnp.all(jnp.isfinite(positions)),
                 lambda: positions,
@@ -781,7 +776,7 @@ def make_microscope_data(
                 ),
             )
 
-        def check_wavelength():
+        def check_wavelength() -> Float[Array, " "]:
             return lax.cond(
                 wavelength > 0,
                 lambda: wavelength,
@@ -790,19 +785,25 @@ def make_microscope_data(
                 ),
             )
 
-        def check_dx():
+        def check_dx() -> Float[Array, " "]:
             return lax.cond(
                 dx > 0,
                 lambda: dx,
                 lambda: lax.stop_gradient(lax.cond(False, lambda: dx, lambda: dx)),
             )
 
-        def check_consistency():
-            P = positions.shape[0]
+        def check_consistency() -> Tuple[
+            Union[Float[Array, "P H W"], Float[Array, "X Y H W"]],
+            Num[Array, " P 2"],
+        ]:
+            pp = positions.shape[0]
 
-            def check_3d_consistency():
+            def check_3d_consistency() -> Tuple[
+                Union[Float[Array, "pp H W"], Float[Array, "X Y H W"]],
+                Num[Array, " pp 2"],
+            ]:
                 return lax.cond(
-                    image_data.shape[0] == P,
+                    image_data.shape[0] == pp,
                     lambda: (image_data, positions),
                     lambda: lax.stop_gradient(
                         lax.cond(
@@ -813,9 +814,12 @@ def make_microscope_data(
                     ),
                 )
 
-            def check_4d_consistency():
+            def check_4d_consistency() -> Tuple[
+                Union[Float[Array, "P H W"], Float[Array, "X Y H W"]],
+                Num[Array, " P 2"],
+            ]:
                 return lax.cond(
-                    image_data.shape[0] * image_data.shape[1] == P,
+                    image_data.shape[0] * image_data.shape[1] == pp,
                     lambda: (image_data, positions),
                     lambda: lax.stop_gradient(
                         lax.cond(
@@ -827,7 +831,9 @@ def make_microscope_data(
                 )
 
             return lax.cond(
-                image_data.ndim == 3, check_3d_consistency, check_4d_consistency
+                image_data.ndim == expected_image_dim,
+                check_3d_consistency,
+                check_4d_consistency,
             )
 
         check_image_dimensions()
@@ -855,32 +861,31 @@ def make_diffractogram(
     wavelength: scalar_float,
     dx: scalar_float,
 ) -> Diffractogram:
-    """
-    Description
-    -----------
-    JAX-safe factory function for Diffractogram with data validation.
+    """JAX-safe factory function for Diffractogram with data validation.
 
     Parameters
     ----------
-    - `image` (Float[Array, "H W"]):
+    image : Float[Array, "H W"]
         Image data
-    - `wavelength` (scalar_float):
+    wavelength : scalar_float
         Wavelength of the optical wavefront in meters
-    - `dx` (scalar_float):
+    dx : scalar_float
         Spatial sampling interval (grid spacing) in meters
 
     Returns
     -------
-    - `diffractogram` (Diffractogram):
+    Diffractogram
         Validated diffractogram instance
 
     Raises
     ------
-    - ValueError:
+    ValueError
         If data is invalid or parameters are out of valid ranges
 
-    Flow
-    ----
+    Notes
+    -----
+    Algorithm:
+
     - Convert inputs to JAX arrays
     - Validate image array:
         - Check it's 2D
@@ -890,21 +895,22 @@ def make_diffractogram(
         - Check dx is positive
     - Create and return Diffractogram instance
     """
-    image = jnp.asarray(image, dtype=jnp.float64)
-    wavelength = jnp.asarray(wavelength, dtype=jnp.float64)
-    dx = jnp.asarray(dx, dtype=jnp.float64)
+    image: Float[Array, " H W"] = jnp.asarray(image, dtype=jnp.float64)
+    wavelength: Float[Array, " "] = jnp.asarray(wavelength, dtype=jnp.float64)
+    dx: Float[Array, " "] = jnp.asarray(dx, dtype=jnp.float64)
+    expected_sample_dim: int = 2
 
-    def validate_and_create():
-        def check_2d_image():
+    def validate_and_create() -> Diffractogram:
+        def check_2d_image() -> Float[Array, " H W"]:
             return lax.cond(
-                image.ndim == 2,
+                image.ndim == expected_sample_dim,
                 lambda: image,
                 lambda: lax.stop_gradient(
                     lax.cond(False, lambda: image, lambda: image)
                 ),
             )
 
-        def check_image_finite():
+        def check_image_finite() -> Float[Array, " H W"]:
             return lax.cond(
                 jnp.all(jnp.isfinite(image)),
                 lambda: image,
@@ -913,7 +919,7 @@ def make_diffractogram(
                 ),
             )
 
-        def check_image_nonnegative():
+        def check_image_nonnegative() -> Float[Array, " H W"]:
             return lax.cond(
                 jnp.all(image >= 0),
                 lambda: image,
@@ -922,7 +928,7 @@ def make_diffractogram(
                 ),
             )
 
-        def check_wavelength():
+        def check_wavelength() -> Float[Array, " "]:
             return lax.cond(
                 wavelength > 0,
                 lambda: wavelength,
@@ -931,7 +937,7 @@ def make_diffractogram(
                 ),
             )
 
-        def check_dx():
+        def check_dx() -> Float[Array, " "]:
             return lax.cond(
                 dx > 0,
                 lambda: dx,
@@ -958,30 +964,29 @@ def make_sample_function(
     sample: Complex[Array, "H W"],
     dx: scalar_float,
 ) -> SampleFunction:
-    """
-    Description
-    -----------
-    JAX-safe factory function for SampleFunction with data validation.
+    """JAX-safe factory function for SampleFunction with data validation.
 
     Parameters
     ----------
-    - `sample` (Complex[Array, "H W"]):
+    sample : Complex[Array, "H W"]
         The sample function
-    - `dx` (scalar_float):
+    dx : scalar_float
         Spatial sampling interval (grid spacing) in meters
 
     Returns
     -------
-    - `sample_function` (SampleFunction):
+    SampleFunction
         Validated sample function instance
 
     Raises
     ------
-    - ValueError:
+    ValueError
         If data is invalid or parameters are out of valid ranges
 
-    Flow
-    ----
+    Notes
+    -----
+    Algorithm:
+
     - Convert inputs to JAX arrays
     - Validate sample array:
         - Check it's 2D
@@ -990,20 +995,21 @@ def make_sample_function(
         - Check dx is positive
     - Create and return SampleFunction instance
     """
-    sample = jnp.asarray(sample, dtype=jnp.complex128)
-    dx = jnp.asarray(dx, dtype=jnp.float64)
+    sample: Complex[Array, "H W"] = jnp.asarray(sample, dtype=jnp.complex128)
+    dx: Float[Array, " "] = jnp.asarray(dx, dtype=jnp.float64)
+    expected_sample_dim: int = 2
 
-    def validate_and_create():
-        def check_2d_sample():
+    def validate_and_create() -> SampleFunction:
+        def check_2d_sample() -> Complex[Array, "H W"]:
             return lax.cond(
-                sample.ndim == 2,
+                sample.ndim == expected_sample_dim,
                 lambda: sample,
                 lambda: lax.stop_gradient(
                     lax.cond(False, lambda: sample, lambda: sample)
                 ),
             )
 
-        def check_sample_finite():
+        def check_sample_finite() -> Complex[Array, "H W"]:
             return lax.cond(
                 jnp.all(jnp.isfinite(sample)),
                 lambda: sample,
@@ -1012,7 +1018,7 @@ def make_sample_function(
                 ),
             )
 
-        def check_dx():
+        def check_dx() -> scalar_float:
             return lax.cond(
                 dx > 0,
                 lambda: dx,
