@@ -1,7 +1,4 @@
-"""
-Module: tools.optimizers
------------------------
-Complex-valued optimizers with Wirtinger derivatives for ptychography.
+"""Complex-valued optimizers with Wirtinger derivatives for ptychography.
 
 This module implements complex-valued optimization algorithms including Adam,
 Adagrad, and RMSprop using Wirtinger calculus. It also provides learning rate
@@ -10,51 +7,51 @@ support automatic differentiation.
 
 Classes
 -------
-- `LRSchedulerState`:
+LRSchedulerState
     State maintained by learning rate schedulers
-- `OptimizerState`:
+OptimizerState
     State maintained by optimizers (moments, step count)
-- `Optimizer`:
+Optimizer
     Optimizer configuration with init and update functions
 
 Scheduler Functions
-------------------
-- `create_cosine_scheduler`:
+-------------------
+create_cosine_scheduler
     Creates a cosine learning rate scheduler with smooth decay
-- `create_step_scheduler`:
+create_step_scheduler
     Creates a step decay scheduler with periodic learning rate drops
-- `create_warmup_cosine_scheduler`:
+create_warmup_cosine_scheduler
     Creates a scheduler with linear warmup followed by cosine decay
-- `init_scheduler_state`:
+init_scheduler_state
     Initialize scheduler state with given learning rate
 
 Optimizer Functions
-------------------
-- `wirtinger_grad`:
+-------------------
+wirtinger_grad
     Compute the Wirtinger gradient of a complex-valued function
-- `complex_adam`:
+complex_adam
     Complex-valued Adam optimizer based on Wirtinger derivatives
-- `complex_adagrad`:
+complex_adagrad
     Complex-valued Adagrad optimizer based on Wirtinger derivatives
-- `complex_rmsprop`:
+complex_rmsprop
     Complex-valued RMSprop optimizer based on Wirtinger derivatives
 
 Initialization Functions
------------------------
-- `init_adam`:
+------------------------
+init_adam
     Initialize Adam optimizer state
-- `init_adagrad`:
+init_adagrad
     Initialize Adagrad optimizer state
-- `init_rmsprop`:
+init_rmsprop
     Initialize RMSprop optimizer state
 
 Update Functions
----------------
-- `adam_update`:
+----------------
+adam_update
     Update parameters using Adam optimizer with Wirtinger derivatives
-- `adagrad_update`:
+adagrad_update
     Update parameters using Adagrad optimizer with Wirtinger derivatives
-- `rmsprop_update`:
+rmsprop_update
     Update parameters using RMSprop optimizer with Wirtinger derivatives
 
 Notes
@@ -73,18 +70,15 @@ from jaxtyping import Array, Complex, Float
 
 
 class LRSchedulerState(NamedTuple):
-    """
-    Description
-    -----------
-    State maintained by learning rate schedulers.
+    """State maintained by learning rate schedulers.
 
     Attributes
     ----------
-    - `step` (int):
+    step : int
         Current optimization step
-    - `learning_rate` (float):
+    learning_rate : float
         Current learning rate
-    - `initial_lr` (float):
+    initial_lr : float
         Initial learning rate value
     """
 
@@ -314,9 +308,9 @@ class Optimizer(NamedTuple):
 
 
 def wirtinger_grad(
-    func2diff: Callable[..., Float[Array, "..."]],
+    func2diff: Callable[..., Float[Array, " ..."]],
     argnums: Optional[Union[int, Sequence[int]]] = 0,
-) -> Callable[..., Union[Complex[Array, "..."], Tuple[Complex[Array, "..."], ...]]]:
+) -> Callable[..., Union[Complex[Array, " ..."], Tuple[Complex[Array, " ..."], ...]]]:
     """
     Description
     -----------
@@ -329,7 +323,7 @@ def wirtinger_grad(
 
     Parameters
     ----------
-    - `func2diff` (Callable[..., Float[Array, "..."]]):
+    - `func2diff` (Callable[..., Float[Array, " ..."]]):
         A complex-valued function to differentiate.
     - `argnums` (Union[int, Sequence[int]]):
         Specifies which argument(s) to compute the gradient with respect to.
@@ -337,15 +331,15 @@ def wirtinger_grad(
 
     Returns
     -------
-    - grad_f (Callable[..., Union[Complex[Array, "..."],
-              Tuple[Complex[Array, "..."], ...]]]):
+    - grad_f (Callable[..., Union[Complex[Array, " ..."],
+              Tuple[Complex[Array, " ..."], ...]]]):
         A function that computes the Wirtinger gradient of f with respect to
         the specified argument(s).
     """
 
     def grad_f(
         *args: Any,
-    ) -> Union[Complex[Array, "..."], Tuple[Complex[Array, "..."], ...]]:
+    ) -> Union[Complex[Array, " ..."], Tuple[Complex[Array, " ..."], ...]]:
         def split_complex(args):
             return tuple(
                 jnp.real(arg) if jnp.iscomplexobj(arg) else arg for arg in args
@@ -357,7 +351,7 @@ def wirtinger_grad(
         def combine_complex(r, i):
             return tuple(
                 rr + 1j * ii if jnp.iscomplexobj(arg) else rr
-                for rr, ii, arg in zip(r, i, args)
+                for rr, ii, arg in zip(r, i, args, strict=False)
             )
 
         split_args = split_complex(args)
@@ -374,22 +368,21 @@ def wirtinger_grad(
 
         if isinstance(argnums, int):
             return 0.5 * (gr - 1j * gi)
-        else:
-            return tuple(0.5 * (grr - 1j * gii) for grr, gii in zip(gr, gi))
+        return tuple(0.5 * (grr - 1j * gii) for grr, gii in zip(gr, gi, strict=False))
 
     return grad_f
 
 
 def complex_adam(
-    params: Complex[Array, "..."],
-    grads: Complex[Array, "..."],
-    state: Tuple[Complex[Array, "..."], Complex[Array, "..."], int],
+    params: Complex[Array, " ..."],
+    grads: Complex[Array, " ..."],
+    state: Tuple[Complex[Array, " ..."], Complex[Array, " ..."], int],
     learning_rate: float = 0.001,
     beta1: float = 0.9,
     beta2: float = 0.999,
     eps: float = 1e-8,
 ) -> Tuple[
-    Complex[Array, "..."], Tuple[Complex[Array, "..."], Complex[Array, "..."], int]
+    Complex[Array, " ..."], Tuple[Complex[Array, " ..."], Complex[Array, " ..."], int]
 ]:
     """
     Description
@@ -401,11 +394,11 @@ def complex_adam(
 
     Parameters
     ----------
-    - `params` (Complex[Array, "..."]):
+    - `params` (Complex[Array, " ..."]):
         Current complex-valued parameters
-    - `grads` (Complex[Array, "..."]):
+    - `grads` (Complex[Array, " ..."]):
         Complex-valued gradients computed using Wirtinger derivatives
-    - `state` (Tuple[Complex[Array, "..."], Complex[Array, "..."], int]):
+    - `state` (Tuple[Complex[Array, " ..."], Complex[Array, " ..."], int]):
         Optimizer state containing (first moment, second moment, timestep)
     - `learning_rate` (float):
         Learning rate for parameter updates.
@@ -422,9 +415,9 @@ def complex_adam(
 
     Returns
     -------
-    - `new_params` (Complex[Array, "..."]):
+    - `new_params` (Complex[Array, " ..."]):
         Updated complex-valued parameters
-    - `new_state` (Tuple[Complex[Array, "..."], Complex[Array, "..."], int]):
+    - `new_state` (Tuple[Complex[Array, " ..."], Complex[Array, " ..."], int]):
         Updated optimizer state
 
     Flow
@@ -449,12 +442,12 @@ def complex_adam(
 
 
 def complex_adagrad(
-    params: Complex[Array, "..."],
-    grads: Complex[Array, "..."],
-    state: Complex[Array, "..."],
+    params: Complex[Array, " ..."],
+    grads: Complex[Array, " ..."],
+    state: Complex[Array, " ..."],
     learning_rate: float = 0.01,
     eps: float = 1e-8,
-) -> Tuple[Complex[Array, "..."], Complex[Array, "..."]]:
+) -> Tuple[Complex[Array, " ..."], Complex[Array, " ..."]]:
     """
     Description
     -----------
@@ -465,11 +458,11 @@ def complex_adagrad(
 
     Parameters
     ----------
-    - `params` (Complex[Array, "..."]):
+    - `params` (Complex[Array, " ..."]):
         Current complex-valued parameters
-    - `grads` (Complex[Array, "..."]):
+    - `grads` (Complex[Array, " ..."]):
         Complex-valued gradients computed using Wirtinger derivatives
-    - `state` (Complex[Array, "..."]):
+    - `state` (Complex[Array, " ..."]):
         Optimizer state containing accumulated squared gradients
     - `learning_rate` (float):
         Learning rate for parameter updates.
@@ -480,9 +473,9 @@ def complex_adagrad(
 
     Returns
     -------
-    - `new_params` (Complex[Array, "..."]):
+    - `new_params` (Complex[Array, " ..."]):
         Updated complex-valued parameters
-    - `new_state` (Complex[Array, "..."]):
+    - `new_state` (Complex[Array, " ..."]):
         Updated optimizer state with accumulated gradients
 
     Flow
@@ -507,13 +500,13 @@ def complex_adagrad(
 
 
 def complex_rmsprop(
-    params: Complex[Array, "..."],
-    grads: Complex[Array, "..."],
-    state: Complex[Array, "..."],
+    params: Complex[Array, " ..."],
+    grads: Complex[Array, " ..."],
+    state: Complex[Array, " ..."],
     learning_rate: float = 0.001,
     decay_rate: float = 0.9,
     eps: float = 1e-8,
-) -> Tuple[Complex[Array, "..."], Complex[Array, "..."]]:
+) -> Tuple[Complex[Array, " ..."], Complex[Array, " ..."]]:
     """
     Description
     -----------
@@ -524,11 +517,11 @@ def complex_rmsprop(
 
     Parameters
     ----------
-    - `params` (Complex[Array, "..."]):
+    - `params` (Complex[Array, " ..."]):
         Current complex-valued parameters
-    - `grads` (Complex[Array, "..."]):
+    - `grads` (Complex[Array, " ..."]):
         Complex-valued gradients computed using Wirtinger derivatives
-    - `state` (Complex[Array, "..."]):
+    - `state` (Complex[Array, " ..."]):
         Optimizer state containing moving average of squared gradients
     - `learning_rate` (float):
         Learning rate for parameter updates.
@@ -542,9 +535,9 @@ def complex_rmsprop(
 
     Returns
     -------
-    - `new_params` (Complex[Array, "..."]):
+    - `new_params` (Complex[Array, " ..."]):
         Updated complex-valued parameters
-    - `new_state` (Complex[Array, "..."]):
+    - `new_state` (Complex[Array, " ..."]):
         Updated optimizer state with moving average
 
     Flow
@@ -626,14 +619,14 @@ def init_rmsprop(shape: tuple) -> OptimizerState:
 
 
 def adam_update(
-    params: Complex[Array, "..."],
-    grads: Complex[Array, "..."],
+    params: Complex[Array, " ..."],
+    grads: Complex[Array, " ..."],
     state: OptimizerState,
     learning_rate: float = 0.001,
     beta1: float = 0.9,
     beta2: float = 0.999,
     eps: float = 1e-8,
-) -> tuple[Complex[Array, "..."], OptimizerState]:
+) -> tuple[Complex[Array, " ..."], OptimizerState]:
     """
     Description
     -----------
@@ -641,9 +634,9 @@ def adam_update(
 
     Parameters
     ----------
-    - `params` (Complex[Array, "..."]):
+    - `params` (Complex[Array, " ..."]):
         Current complex-valued parameters
-    - `grads` (Complex[Array, "..."]):
+    - `grads` (Complex[Array, " ..."]):
         Complex-valued gradients computed using Wirtinger derivatives
     - `state` (OptimizerState):
         Current optimizer state
@@ -662,7 +655,7 @@ def adam_update(
 
     Returns
     -------
-    - `new_params` (Complex[Array, "..."]):
+    - `new_params` (Complex[Array, " ..."]):
         Updated complex-valued parameters
     - `new_state` (OptimizerState):
         Updated optimizer state
@@ -681,12 +674,12 @@ def adam_update(
 
 
 def adagrad_update(
-    params: Complex[Array, "..."],
-    grads: Complex[Array, "..."],
+    params: Complex[Array, " ..."],
+    grads: Complex[Array, " ..."],
     state: OptimizerState,
     learning_rate: float = 0.01,
     eps: float = 1e-8,
-) -> tuple[Complex[Array, "..."], OptimizerState]:
+) -> tuple[Complex[Array, " ..."], OptimizerState]:
     """
     Description
     -----------
@@ -694,9 +687,9 @@ def adagrad_update(
 
     Parameters
     ----------
-    - `params` (Complex[Array, "..."]):
+    - `params` (Complex[Array, " ..."]):
         Current complex-valued parameters
-    - `grads` (Complex[Array, "..."]):
+    - `grads` (Complex[Array, " ..."]):
         Complex-valued gradients computed using Wirtinger derivatives
     - `state` (OptimizerState):
         Current optimizer state
@@ -709,7 +702,7 @@ def adagrad_update(
 
     Returns
     -------
-    - `new_params` (Complex[Array, "..."]):
+    - `new_params` (Complex[Array, " ..."]):
         Updated complex-valued parameters
     - `new_state` (OptimizerState):
         Updated optimizer state
@@ -726,13 +719,13 @@ def adagrad_update(
 
 
 def rmsprop_update(
-    params: Complex[Array, "..."],
-    grads: Complex[Array, "..."],
+    params: Complex[Array, " ..."],
+    grads: Complex[Array, " ..."],
     state: OptimizerState,
     learning_rate: float = 0.001,
     decay_rate: float = 0.9,
     eps: float = 1e-8,
-) -> tuple[Complex[Array, "..."], OptimizerState]:
+) -> tuple[Complex[Array, " ..."], OptimizerState]:
     """
     Description
     -----------
@@ -740,9 +733,9 @@ def rmsprop_update(
 
     Parameters
     ----------
-    - `params` (Complex[Array, "..."]):
+    - `params` (Complex[Array, " ..."]):
         Current complex-valued parameters
-    - `grads` (Complex[Array, "..."]):
+    - `grads` (Complex[Array, " ..."]):
         Complex-valued gradients computed using Wirtinger derivatives
     - `state` (OptimizerState):
         Current optimizer state
@@ -758,7 +751,7 @@ def rmsprop_update(
 
     Returns
     -------
-    - `new_params` (Complex[Array, "..."]):
+    - `new_params` (Complex[Array, " ..."]):
         Updated complex-valued parameters
     - `new_state` (OptimizerState):
         Updated optimizer state
